/***************************************************************************************

Basic Lite-C demo for MDJK_A7 Pathfinding DLL

***** "DIJKSTRA" Algo & WED - *****
*****  Main Test file         *****

Nodes and edges are edited in WED using path Objects (Add->path and Vertex Move mode)

Version 0.5 - By MadJack 17 Juillet 2008
****************************************************************************************/
///////////////////////////////
#define PRAGMA_PATH "%EXE_DIR%\templates\images";
#define PRAGMA_PATH "%EXE_DIR%\templates\models";
#define PRAGMA_PATH "%EXE_DIR%\templates\sounds";

#include <acknex.h>
#include <default.c>
#include "mtlFX.c"
///////////////////////////////


#include "MDJK_A7.h"									// This header defines DLL functions prototypes
#include "DJ_Test.h"
#include "move.c"

// GLOBAL VARIABLES

var PtDestChief;				
var PtPlotChief;				
VECTOR PlotChief;
var ChiefIsSet;

//***************************************************************************************

function main()
{
	ChiefIsSet = 0;
	video_switch (8, 32, 0);
	
	level_load("Level.WMB");
	
	wait(10);
	
}

//uses: IS_CHIEF
action guardPatrol()
{
	VECTOR vFeet;
	char* myPath = "path_000";
	
	// Some initialisation...
	
	my.NB_BOUNCES 	= 0;
	my.PANIC			= 0;
	
	my.GUARD_SPEED = 5 + random (2);				// different speed for each guard
	my.GUARD_ANIM_PERCENT =0;
	my.CHOC = 0;
	
	vec_for_min(vFeet,me); 
	c_setminmax(me); 
	
	my.GUARD_V_FEET = vFeet.z;						// vertical correction
	
	my.emask |= (ENABLE_BLOCK | ENABLE_ENTITY|IGNORE_PASSABLE); // make entity sensitive for block and entity collision
	my.event = bounce_event;
	
	if(is(my,IS_CHIEF)) {
		if (ChiefIsSet == 1) {
			reset(my,IS_CHIEF); // There is only one chief !!!
		}
		else ChiefIsSet = 1;
	} 
	
	//****************************************  DLL STEP 1 - path_MDJK_init  ********************************************
	
	my.NB_NODES = path_MDJK_init(me, _chr (myPath));			// Attach Entity to the named path and prepare the initial matrix 
																				// Returns the number of nodes in the path
	//*******************************************************************************************************************
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// Once Entity is attached to the path, you can use 2 functions in this version
	//
	//	path_MDJK_find(me, PtDep, PtDest);			which calculate the path from PtDep to PtDest
	// path_MDJK_next_point(me, PtOn) 				on the way to PtDest, we are under PtOn, so what is the next point ?
	//
	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	wait(1);
	
	

	initPos();
	
	while (1)
	{
		//wait(-1);
		
		//******************************************  DLL STEP 2 - path_MDJK_find  ******************************************
		
		path_MDJK_find(me, my.PtDep, my.PtDest);							// Find a path from PtDep to PtDest
		
		//*******************************************************************************************************************
		
		my.PtOn = 0;													// PtOn is where I am (somewhere between PtDep and PtDest)
		
		my.PtNext = my.PtDep;
		
		if (is(my,IS_CHIEF)) movePlot(me, my.PtDest);		// To move the yellow marker (only for the chief)
		
		while (my.PtNext>0)											// PtNext is positive until we reach PtDep
		{
			
			my.PtOn = my.PtNext;
			
			path_getnode(me,my.PtNext,me.NEXT_POINT_POS_X, NULL);
			
			// Between two points of the path :
			while (moveToNextNode()) 			//	moveToNextNode() return positive value untill the PtNext point
			{
				wait(1);
			}		
			my.NB_BOUNCES = 0;
			if (my.PANIC == 1) initPos();
			
			else {
				//*************************************  DLL STEP 3 - path_MDJK_next_point  ************************************
				// PtNext = path_MDJK_next_point(me,PtOn);								// DLL Most useful function :
				// Coming from PtDep,I am near PtOn, on the way to PtDest, 
				// What is the next point ?
				// Return zero if there is no solution or if PtOn=PtDest
				//**************************************************************************************************************
				my.PtNext = path_MDJK_next_point(me, my.PtOn);
			}
			my.PANIC = 0;
		}
		my.PtDep = my.PtDest;
		my.PtDest = integer (random (my.NB_NODES)) + 1;
		if (is(my,IS_CHIEF)) PtDestChief = my.PtDest;
		
	}
	
}

////////////////////////////////////////////////////////////////////////////////
// walking from PtOn to PtNext
//
var moveToNextNode()
{
	// to move, the guard we'll :
	// 	- rotate toward PtNext : 							guardRotate				STEP1
	// 	- move forward : 										guardMove				STEP2
	// 	- animate :												guardAnimate			STEP3
	//
	// if this guard is the chief, camera follow him	camRotate				STEP4
	//
	// finally, we test if he has reached PtNext			guardOnTheWay  		STEP5 
	//********************************************************************
	
	// STEP1
	if (my.CHOC <= 0) guardRotate();							// will rotate unless there is something to avoid
	
	// STEP2
	guardMove();
	
	// STEP3													
	guardAnimate("walk");
	
	// STEP4
	if (is(my,IS_CHIEF)) camRotate();						// camera following the chief
	
	
	if(my.CHOC >0) my.CHOC --;									// my.CHOC is a variable active for a few cycles
	else my.CHOC = 0;												// we keep my.CHOC >= 0
	
	
	// STEP5
	//
	// my.PANIC is set if the guard "feels" lost
	// 
	// guardOnTheWay (X) returns 1 if the distance from the guard to PtNext is more than X quants
	// N.B.: uses only x and y, equivalent to : Guard is "under" PtNext 
	//
	return (guardOnTheWay (50) * (my.PANIC==0));			// if the level is larger, a bigger value will smooth trajectory		

}

////////////////////////////////////////////////////////////////////////////////
// prepare yellow marker move
//
void movePlot(ENTITY* entPtr, int aDest)
{
	
	path_getnode(entPtr,aDest,PlotChief.x, NULL);

}

////////////////////////////////////////////////////////////////////////////////
// Yellow marker
//
action showDestPlot(ENTITY* entPtr)
{
	var locZ= my.z;
	
	while (1)
	{
		c_trace(my.x,vector(my.x,my.y,my.z-5000),IGNORE_ME | USE_BOX);
		
		draw_line3d(target.x, vector(0,0,255), 0);
		if (PtPlotChief != PtDestChief)
		{
			my.x = PlotChief.x;
			my.y = PlotChief.y;
			my.z = PlotChief.z;
			
			PtPlotChief=PtDestChief;
		}
		draw_line3d(my.x, vector(203,251,11), 100);
		wait(1);
	}
	
}

////////////////////////////////////////////////////////////////////////////////
// Small event management for avoidance
//
void bounce_event() 
{
	var rot, rotmax, cycle;
	
	switch (event_type)
	{
		case EVENT_BLOCK:								// block on the way														
		
		my.NB_BOUNCES ++;								// we count these impacts
		
		if (my.NB_BOUNCES < 50) 					// between 2 points, we admit 50 impacts before PANIC
		{
			
			if ((target.z < my.z + 40) ) {		// it is a low impact => stairs
				return;
			}
			
			my.CHOC = 5;								
			if(my.PtDest%2) my.pan += 20;	
			else my.pan -= 20;
			
		}
		else {
			my.NB_BOUNCES = 0;						// too much impacts, guard is lost => PANIC
			my.CHOC = 0;
			my.PANIC = 1;
		}
		break;
		
		case EVENT_ENTITY:							// entity on the way
		
		rot = random(10);								// some random choice (more realistic)			
		cycle = random (5) + 1;
		
		my.pan -= rot;
		my.CHOC = cycle;
		
		break;
		
		
	}
}

////////////////////////////////////////////////////////////////////////////////
// Used at start and when PANIC is ON
//
void initPos()
{
	my.PtNext = path_scan(me,my.x,my.pan,vector(360,180,1000));
	my.PtOn = 0;										
	my.PtDep = my.PtDest = my.PtNext;
	
}