#include <transform>
#include <fog>
#include <pos>
#include <normal>
#include <tangent>
#include <light>

float4 vecLight;

texture entSkin1;	// texture
texture entSkin2;	// normal map

float4 vecSkill41;
// vecSkill41.x: float facAmbient = 0,5;
// vecSkill41.y: float facDiff = 1.5;

sampler sBaseTex = sampler_state { Texture = <entSkin1>; MipFilter = Linear;	};
sampler sBump = sampler_state { Texture = <entSkin2>; MipFilter = Linear;	};

struct bumpOut
{
	float4 Pos:    	POSITION;
	float  Fog:			FOG;
	float4 Color:		COLOR;		
	float4 Ambient:	COLOR1;
	float2 Tex:   		TEXCOORD0;	
	float3 Position:	TEXCOORD1;	
	float3 Light:		TEXCOORD2;	
	float3 Normal: 	TEXCOORD3;	
	float3 Tangent:	TEXCOORD4;
};

bumpOut perPixel_VS(
in float4 inPos: 		POSITION, 
in float3 inNormal:	NORMAL,
in float2 inTex: 		TEXCOORD0,
in float3 inTangent: TEXCOORD1)
{
	bumpOut Out;

	Out.Pos	= DoTransform(inPos);
	Out.Tex	= inTex;
	Out.Fog	= DoFog(inPos);
	
	float facAmbient = vecSkill41.x / 100.;
	Out.Ambient = facAmbient*vecLight;	
	
	Out.Normal = inNormal;
	Out.Tangent = inTangent;
	
	Out.Position = inPos;
	
	float3 P = DoPos(inPos);
	
	Out.Light = vecLightPos[0].xyz;
	
	float fDist = length(vecLightPos[0].xyz - P)/vecLightPos[0].w;
	if (vecLightPos[0].w>9000)	fDist = 0.;
   if (fDist < 1.f)	fDist = 1.f - fDist;
   else	fDist = 0.f;
   
	Out.Color = fDist*vecLightColor[0];
	
	return Out;		
}


float4 diffuseBump_PS(bumpOut In): COLOR
{
	float4 base = tex2D(sBaseTex,In.Tex);
	float3 bumpNormal = tex2D(sBump,In.Tex)*2-1;
	
	CreateTangents(In.Normal,In.Tangent);
	float3 lightDir = DoTangent(normalize(In.Light-In.Position));
		
	float facDiff = vecSkill41.y / 20.;
	float diffuse = saturate(dot(lightDir,bumpNormal)) * facDiff;
	
	return base * (diffuse*In.Color + In.Ambient);
}

technique bump
{
	pass One
	{		
		VertexShader = compile vs_2_0 perPixel_VS();
		PixelShader = compile ps_2_0 diffuseBump_PS();
	}
}

technique fallback { pass one { } }