#include <transform>
#include <fog>
#include <pos>
#include <normal>
#include <tangent>
#include <light>

float4 vecLight;

texture entSkin1;	// texture
texture entSkin2;	// normalmap with heightmap on alpha-channel

const float facAmbient = 0.2;
float4 vecSkill41;
//vecSkill41.x: float facDiff = 0.5;
//vecSkill41.y: float facSpec = 0.8;
//vecSkill41.z: float shininess = 4;
//vecSkill41.w: float facParallax = 0.025;

sampler sBaseTex = sampler_state { Texture = <entSkin1>; MipFilter = Linear;	};
sampler sBump = sampler_state { Texture = <entSkin2>; MipFilter = Linear;	};

struct paraOut
{
	float4 Pos:    	POSITION;
	float  Fog:			FOG;
	float4 Color:		COLOR;
	float4 Ambient:	COLOR1;
	float2 Tex:   		TEXCOORD0;
	float3 Position:  TEXCOORD1;
	float3 Light:		TEXCOORD2;	
	float3 Normal: 	TEXCOORD3;	
	float3 Tangent:	TEXCOORD4;
};

paraOut perPixel_VS(
in float4 inPos: 		POSITION, 
in float3 inNormal:	NORMAL,
in float2 inTex: 		TEXCOORD0,
in float3 inTangent: TEXCOORD1)
{
	paraOut Out;

	Out.Pos	= DoTransform(inPos);
	Out.Tex	= inTex;
	Out.Fog	= DoFog(inPos);
	Out.Ambient = facAmbient*vecLight;	
	
	Out.Normal = inNormal;
	Out.Tangent = inTangent;
	
	Out.Position = inPos;
	
	Out.Light = vecLightPos[0].xyz;
	
	float3 P = DoPos(inPos);
	
	float fDist = length(vecLightPos[0].xyz - P)/vecLightPos[0].w;
	if (vecLightPos[0].w>9000)	fDist = 0.;
   if (fDist < 1.f)	fDist = 1.f - fDist;
   else	fDist = 0.f;
   
	Out.Color = fDist*vecLightColor[0];
	
	return Out;		
}


float4 phongPOM_PS(paraOut In): COLOR
{
	
	CreateTangents(In.Normal,In.Tangent);
	float3 viewDir = DoTangent( normalize(vecViewPos-In.Position) );
	float facParallax = vecSkill41.w / 4000.;
	viewDir.z *= -1;	viewDir *= facParallax/viewDir.z;
	
	const int steps=15;
	
	float stepSize = 1.0/steps;

	float actStep = 0.0;	float height;	float lastHeight;

	for( int i=0;i<steps-1;i++ )
	{
		height = tex2D(sBump,In.Tex+viewDir.xy*actStep).w;
		if (actStep<height)
		{
			actStep += stepSize;
			lastHeight = height;
		}
		else
			i=steps;
	}
	
	float finalParaLength = ( actStep - stepSize * ( 1 - ( (actStep-stepSize-lastHeight) / (height-stepSize-lastHeight) ) ) );
	
	float2 newTexCoord = In.Tex + viewDir.xy * finalParaLength;
	
	float4 base = tex2D(sBaseTex,newTexCoord);
	float3 paraNormal = tex2D(sBump,newTexCoord).xyz*2-1;
	
	float3 lightDir = DoTangent( normalize(In.Light-In.Position) );
	
	float facSpec = vecSkill41.y / 20.;
	float shininess;
	if ( vecSkill41.w <= 20. )
		shininess = vecSkill41.z / 20.;
	else
		shininess = (vecSkill41.z-19)*2.;
	float spec = pow(max(dot(lightDir,paraNormal),0),shininess) * facSpec;	
	
	float facDiff = vecSkill41.x / 20.;
	float diffuse = saturate(dot(lightDir,paraNormal)) * facDiff;
	
	lightDir.z *= -1;	lightDir *= facParallax/lightDir.z;
	
	actStep = 0.0;

	for( int i=0;i<steps-1;i++ )
	{
		height = tex2D(sBump,In.Tex+lightDir.xy*actStep).w;
		if (actStep<height)
		{
			actStep += stepSize;
			lastHeight = height;
		}
		else
			i=steps;
	}
	float finalLightLength = ( actStep - stepSize * ( 1 - ( (actStep-stepSize-lastHeight) / (height-stepSize-lastHeight) ) ) );
	if (finalLightLength<finalParaLength-0.1)
	{
		diffuse /= 3.;
		spec = 0;
	}
	
	return base * ( (diffuse+spec) * In.Color + In.Ambient);
}

technique pom
{
	pass One
	{		
		VertexShader = compile vs_2_0 perPixel_VS();
		PixelShader = compile ps_2_a phongPOM_PS();
	}
}

technique fallback { pass one { } }