/***************************************************************************************

Basic Lite-C  guard movements and camera management

Based on  CONITEC - Tech demonstration


Version 0.5 - By MadJack 17 Juillet 2008
****************************************************************************************///////////////////////////////////////////////////////////////////////


#include "move.h"


void guardMove()
{
	// determine the ground distance by a downwards trace
	var dist_down; 
	if (c_trace(my.x,vector(my.x,my.y,my.z-5000),IGNORE_ME | IGNORE_PASSABLE | USE_BOX) > 0)
	dist_down = my.z + my.GUARD_V_FEET - target.z; // get distance between player's feet and the ground
	else
	dist_down = 0;

	// apply gravity when the player is in the air
	if (dist_down > 0)  // above floor, fall down with increasing speed
	dist_down = clamp(dist_down,0,accelerate(my.GUARD_SPEED_DOWN,5,0.1));
	else                // on or below floor, set downward speed to zero
	my.GUARD_SPEED_DOWN = 0;

	// move the player      
	my.GUARD_DIST = (my.GUARD_SPEED )*time_step;
	my.GUARD_DIST = sign(my.GUARD_DIST)*(abs(my.GUARD_DIST) + 0.3*dist_down); 				// adapt the speed on slopes
	
	c_move(me,vector(my.GUARD_DIST,0,0),vector(0,0,-dist_down),IGNORE_PASSABLE | GLIDE); // move the player
	
}

//--------------------------------------------------------------------
// Rotate guard to the ang
//--------------------------------------------------------------------
void guardRotateAng(var fRotAng, var fRotSpeed) 
{
	var f;
	if (my.pan == fRotAng) { 
		return; 
	}

	f = ang(fRotAng - my.pan);

	if (f > 0) { 
		my.pan += fRotSpeed * time_step; 
	}

	if (f < 0) { 
		my.pan -= fRotSpeed * time_step; 
	}

	if (ang(fRotAng - my.pan) < 0 && f > 0) { 
		my.pan = fRotAng; 
	}

	if (ang(fRotAng - my.pan) > 0 && f < 0) { 
		my.pan = fRotAng; 
	}
}

//--------------------------------------------------------------------
// Rotate the guard
//--------------------------------------------------------------------
void guardRotate()
{
	VECTOR temp[3];
	ANGLE temp2[3];

	vec_set(temp.x, vector(my.NEXT_POINT_POS_X, my.NEXT_POINT_POS_Y, my.NEXT_POINT_POS_Z)); 
	vec_sub(temp.x,my.x);
	vec_to_angle(temp2.pan,temp.x); 
	
	guardRotateAng(temp2.pan,30* time_step); 
	
	my.tilt = 0;
}

//--------------------------------------------------------------------
// Animate the guard
//--------------------------------------------------------------------
void guardAnimate(STRING* sScene)
{
	var fAnimSpeed;
	fAnimSpeed = 1.2*my.GUARD_SPEED;
	
	my.GUARD_ANIM_PERCENT += fAnimSpeed * time_step;
	if (my.GUARD_ANIM_PERCENT < 0) {
		my.GUARD_ANIM_PERCENT += 100;
	} 
	else {
		if (my.GUARD_ANIM_PERCENT > 100) {
			my.GUARD_ANIM_PERCENT -= 100;
		}
	}

	ent_animate(me,"walk",my.GUARD_ANIM_PERCENT,ANM_CYCLE);
	
}
//--------------------------------------------------------------------
// Check point 
//--------------------------------------------------------------------
var guardOnTheWay(var aPrecision)
{
	var dist = 0;
	
	dist = sqrt(pow(my.NEXT_POINT_POS_X-my.x,2)+pow(my.NEXT_POINT_POS_Y-my.y,2));
	
	if (dist <= aPrecision) return FALSE;
	else return TRUE;
}


//--------------------------------------------------------------------
// Rotate the camera
//--------------------------------------------------------------------
void camRotate()
{
	VECTOR temp[3];
	ANGLE *temp2 = vector(0,0,0);

	//set view direction to the next node
	vec_set(temp.x, vector(my.x, my.y, my.z)); 
	vec_sub(temp.x,camera.x);
	vec_to_angle(temp2.pan,temp.x); 
	camRotateAng(temp2,5 * time_step); //65
}
//--------------------------------------------------------------------
// Rotate the camera to the ang
//--------------------------------------------------------------------
void camRotateAng(ANGLE * vectRot, var fRotSpeed) 
{
	var difAng, fRotPan, fRotTilt;
	fRotPan = vectRot.pan;
	fRotTilt = vectRot.tilt;
	var epsilon = 5;
	
	
	if (camera.pan != fRotPan) { 
		difAng = ang(fRotPan - camera.pan);

		if (difAng > 0) { 
			camera.pan += fRotSpeed * time_step; 
			if (ang(fRotPan - camera.pan) < epsilon)  { 
				camera.pan = fRotPan; 
			}
		}

		else { 
			camera.pan -= fRotSpeed * time_step; 
			if (ang(fRotPan - camera.pan) > -epsilon) { 
				camera.pan = fRotPan; 
			} 
		}
		
	}
	if (camera.tilt != fRotTilt) { 
		difAng = ang(fRotTilt - camera.tilt);

		if (difAng > 0) { 
			camera.tilt += fRotSpeed * time_step; 
			if (ang(fRotTilt - camera.tilt) < epsilon)  { 
				camera.tilt = fRotTilt; 
			}
		}

		else { 
			camera.tilt -= fRotSpeed * time_step; 
			if (ang(fRotTilt - camera.tilt) > -epsilon) { 
				camera.tilt = fRotTilt; 
			} 
		}
		
	}
	
}




